/*----------------------------------------------------------------------------
                                INFORMATION    13.03.2022 - 20:10
  ----------------------------------------------------------------------------*/
  
/* Highly customizable wall hook Version 1
   This code enables to create various wall hooks in a higly customizable way. 
   Written by Kakiemon @ Thingiverse
   Rolf Jethon
   Licensed under the GPL v3 Terms and conditions.
   This code includes code from Outward Buckle (Thingiverse) 
   https://www.thingiverse.com/thing:4872188

   HOW TO USE THIS FILE :  I recommend to use the OpenScad Customizer

   -- Solution 1 :
   Open this file with OpenScad software (free and open-source)
   http://www.openscad.org/downloads.html
   Choose your parameters in the SETTINGS chapter (see below)
   Press F5 to compile and see the resulting part
   Check your hook's global size in the console window (below viewing area)
   If OK, press F6 to render
   Go to : File => Export => Export as STL : choose filename and location
   Save the STL file of your customized wall hook
   Send this STL file to your 3D printer

   -- Solution 2 :
   This file had been optimized for the Customizer of OpenSCAD
   OpenScad: Just change the parameters in the customizer section
*/
/*------------------------------------------------------------------------------
With this program you can create wall hooks out of various SVG files and /or out
of text.
==================
 IMPORTANT NOTES:
==================
With this wall hook designer OpenScad reaches nearly its limits. Depending on the
used SVG or text elements, the processing/rendering may take very long. Just 
wait or use more simple files or fonts. As long as the OpenScad process is taking
CPU it is still working!
Text and SVG output may be positive or negative - means it subtracts or it adds
to the wall hook body.  
Parameters textlayer and svglayer with values greater than 0 make the text or
svg solid in any case. If the parameters text_rounding or svg_rounding are
greater than 0, the element has an extension border. 
If with a border the textlayer has a value of 0, only the text border is shown,
this means the text becomes negative.
Some SVGs do not render correctly, because they need to have closed node loops
to be able to extrude them to 3D. Many freely available SVGs have closed loops,
some don't.

------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------
                                SETTINGS          
 ----------------------------------------------------------------------------*/

/* [General] */
// show hidden cutting objects in transparent red
show_cutting=false;//[true,false]
// set the accuracy of the top radii - higher number is more accurate
top_radius_accuracy=200;//[20:200]
// other accuracy segments
$fn=75;//[20:200]
/* [Base Plate] */
// set to true, if you want a base plate
base_enable=true;//[true,false]
// base plate top rounding - enabled if true
base_top_rounding=false;//[true,false]
// base top rounding radius
base_top_radius=50;//[20:.5:200]
// base top shift left or right
base_top_x_shift=0;//[-100:.5:100]
// base top shift up or down
base_top_y_shift=0;//[-100:.5:100]
// diameter (size) of the base plate
basesize=40;//[10:0.5:100]
// rotates te base in the Z-Axis
baserotation=45;//[-180:0:180]
// scale the base in X direction compared to Y
basescale_x=.49;//[0.1:.01:10]
// number of edges the base plate has
baseedges=4;//[3:60]
// rounding radius of the base plate edges
baserounding=12.4;//[0.01:.1:100]
// make an additional border around the base plate
baseborder=2;//[0:.1:30]
// thickness of the base plate 
baseheight=2;//[0:.1:20]
// height of the base plate border - adds to baseheight
baseborderheight=0;//[0:.1:5]
/* [Mounting holes] */
// Set to true, if you want mounting hole(s)
screw_enable=true;//[true,false]
// Height of the mounting hole base
screw_base_thickness=3.5;//[0.1:.1:20]
// Diameter of the mounting screw hole
hole_diameter=4;//[1:.1:11]
// Diameter of the screw head of the mouning screw(s)
screw_head_diameter=9;//[1:.1:18]
// Border around the screw head - adds to screw_head_diameter
screw_head_border=1;//[0:.1:50]
// 45° sunk screw if true - otherwise flat sunk
sunk_screw=false;//[true,false]
// Defines the depht of the sunk inside the base
sunk_z_shift=-1.10;//[-10:.1:20]
// Distance of the two screws in mm - 0 adds only one screw
screw_distance=22;//[0:.1:100]
// shifts the screws in X-direction
screw_x_shift=0;//[-50:.1:50]
// shifts the screws in -direction
screw_y_shift=7;//[-50:.1:50]
// rotates the screw axis
screw_rotate=0;//[-90:90]
/* [Text] */
// enable text output
text_enable = false;//[true,false]
// Text to add
line_01 = "";
line_02 = "";
line_03 = "";
line_04 = "";
line_05 = "";
/* [Text Options] */
// Font for the grill text. Make sure all the text is attached to the hex!
pattern_text_font = "Arial";
// Font size of the text.
pattern_text_size = 8; //[0.5:0.5:100]
// spacing factor in between characters
text_spacing=1;//[0:0.01:2]
// Extra space between text lines
text_line_space = 1; //[-5:0.1:50]
/* [Text Transform Options] */
// round the top of the text body
text_top_rounding=true;//[true,false]
// the thickness of the inner text shape - 0 makes the text negative
textlayer=1;//[0:.1:30]
// the thickness of the outer text shape
text_thickness=5;//[1:0.1:50]
// the complete text top will be rounded by this radius
text_top_radius=20;//[10:.5:200]
// Allows to distort the top radius, to make the text surface ellipsoid
text_top_radius_x_scale=1;//[0.01:0.01:100]
// rotates the text top ellipsoid - only visible if the scale is no 0 
text_top_radius_rotate=0;//[-180:1:180]
// shifts the text top curvature in x direction
text_top_radius_x_shift=0;//[-100:.1:100]
// shifts the text top curvature in x direction
text_top_radius_y_shift=0;//[-100:.1:100]
// Thickness of text outline in mm, set to 0 for a block item, 1 or more for an outline. To optimize for FDM, use a multiple of your line width setting (in your slicer)
text_rounding = 0.5; //[0:0.1:30]
// Move text left/right
text_x_shift = 0; //[-40:0.1:40]
// Move text up/down
text_y_shift = 0; //[-40:0.1:40]
// Rotate text (Inverted if using a flat grill)
text_rotate = 0; //[-180:0.1:180]
// flips the text to wrong reading
text_flip = false;//[true,false]
/* [SVG plate relief] */
// Set to true to enable (Must be using OpenSCAD, will not work in Thingiverse customizer!)
svg_import_enabled = true; //[true,false]
// thickness of the inner SVG shape
svglayer=5.2;//[0:.1:30]
// thickness of the outer SVG shape
svg_thickness=5.1;//[1:0.1:50]
// if the top of the SVG shape shall be rounded
svg_top_rounding=true;//[true,false]
// the radius for the SVG top rounding
svg_top_radius=74;//[10:.5:200]
// other than 0 distorts the top radius shape
svg_top_radius_y_scale=0;//[-1:0.01:.99]
// only visible, if the top_radius_scale is not 0
svg_top_radius_rotate=0;//[-180:1:180]
// shifts the top radius shape left or right
svg_top_radius_x_shift=0;//[-100:.1:100]
// shifts the top radius shape left or right
svg_top_radius_y_shift=-.2;//[-100:.1:100]
// Select the file to import (must be a SVG | set pattern type to svg to use as a cut out. E.G. C:\TempFolder\File.SVG)
file_to_import = "lips.svg";
// Scale the SVG
file_scale = .236; //[0.001:0.001:8]
// Thickness of SVG outline, set to 0 for a block item, 1 or more for an outline
svg_rounding = 0; //[0:0.1:10]
// Move SVG left/right
svg_x_shift = 0; //[-40:0.1:40]
// Move SVG up/down
svg_y_shift = -9.8; //[-40:0.1:40]
// Rotate SVG
svg_rotate = 0; //[-180:0.1:180]
// flip SVG image
svg_flip = false;//[true,false]
/* [SVG Hook] */
// if enabled, you need to provide x-y and x-z svg shapes
enable_svg_hook=false;//[true,false]
// X-Y shape of the hook - select a simple SVG
xy_svg="hooks/xy_base1_1.svg";
// X-Z shape of the hook - select a simple SVG
xz_svg = "hooks/xz_base2.svg";
/* [Standard Hook] */
// enables a standard round hook
enable_standard_hook=true;//[true,false]
// the tip to bottom length of the hook
hook_length=19;//[5:60]
// Hook shaft diameter
hook_shaft_diameter=4.3;//[1:.1:15]
// Hook tapering value
hook_tapering_angle=11;//[-30:.5:60]
// Hook tip ball diameter
hook_ball_diameter=6;//[0:.1:25]
/* [Hook transform options] */
// Defines the size of the resulting hook
svg_hook_scale = .126; //[0.001:0.001:8]
// scales the hook in x direction by this factor
hook_scale_x_factor=1.611;//[0.001:0.001:8]
// Shifts the hook left or right
hook_x_shift =0;//[-100:.1:100]
// Shifts the hook up or down
hook_y_shift =2.7;//[-100:.1:100]
// Shifts the hook in and out
hook_z_shift =0;//[-100:.1:100]
// defines the hook slant to make it a hanger
hook_x_rotate = 13; //[-180:1:180]


/* [Hidden] */
pattern_text_font_val = pattern_text_font;

// Check how many lines are being used
used_line_count =   
                 line_05 != "" ? 05 :
                 line_04 != "" ? 04 :
                 line_03 != "" ? 03 :
                 line_02 != "" ? 02 :
                 line_01 != "" ? 01 : 
                 0;

// Set text_set based on how many lines used
text_set =          
          line_05 != "" ? [line_01, line_02, line_03, line_04, line_05] :
          line_04 != "" ? [line_01, line_02, line_03, line_04] :
          line_03 != "" ? [line_01, line_02, line_03] :
          line_02 != "" ? [line_01, line_02] :
          line_01 != "" ? [line_01] : 
          [];

// calculate some base sizes
svg_topdist=((svg_thickness >= svglayer) && (svg_rounding > 0)) ? svg_thickness:svglayer;
svg_tr_scale=(1/(-svg_top_radius_y_scale+1)/2)/(((-svg_top_radius_y_scale)+1)/2);
text_topdist=((text_thickness >= textlayer) && (text_rounding > 0)) ? text_thickness:textlayer;
br=basesize/2-baserounding > 0.01 ? basesize/2-baserounding:0.01;// this + baserounding = outer diam of plate
ibr=baserounding-baseborder > 0.01 ? baserounding-baseborder:0.01;    // inner border radius
finalborderheight=baseborderheight+baseheight; // final overall height of the plate border
limitedouteroff=basesize/2-br;
limitedinneroff=limitedouteroff-baseborder;
screw_base_diameter=screw_head_diameter+2*screw_head_border;
htd=hook_length*tan(hook_tapering_angle/2)*2+hook_shaft_diameter;
hook_tapered_diameter=htd-.1 < 0 ? .1:htd;

if (screw_enable==true){ // the hook with screws
 difference(){
  union(){   
   do_plate();
   do_text();
   do_svg();
   do_hook(); 
   translate([screw_x_shift,screw_y_shift,0])  
   rotate([0,0,screw_rotate])    
   do_screw_base();   
  }
  translate([screw_x_shift,screw_y_shift,0])
  screw_holes();
  translate([0,0,-99.99]) // cut off everyting below z=0
  cube(200,center=true); 
 } 
} else { // the hook if no screws are needed
  difference(){  
   union(){   
    do_plate();
    do_text();
    do_svg();
    do_hook();
   }
   translate([0,0,-99.99]) // cut off everything below z=0
   cube(200,center=true); 
  } 
}   
// build the screw base foot 
module do_screw_base(){   
 translate([screw_distance/2,0,screw_base_thickness/2])   
 cylinder(h=screw_base_thickness,d=screw_base_diameter,center=true);  
 translate([-screw_distance/2,0,screw_base_thickness/2])   
 cylinder(h=screw_base_thickness,d=screw_base_diameter,center=true);   
}

// build both screw holes and rotate them 
module screw_holes(){
  rotate([0,0,screw_rotate])
  translate([screw_distance/2,0,sunk_z_shift])
  screwhole_core();  
  rotate([0,0,screw_rotate]) 
  translate([-screw_distance/2,0,sunk_z_shift])
  screwhole_core();  
}    

// build the holes including sunk
module screwhole_core(){ 
 if (show_cutting==true){
  #cylinder(h=100,d=hole_diameter,center=true);
  translate([0,0,24.99+screw_head_diameter/4])    
  #cylinder(h=50,d=screw_head_diameter,center=true); 
  if(sunk_screw==true){   
   cylinder(h=screw_head_diameter/2,d1=0,d2=screw_head_diameter,center=true);    
  }   
 } else {
  cylinder(h=100,d=hole_diameter,center=true);   
  translate([0,0,24.99+screw_head_diameter/4])   
  cylinder(h=50,d=screw_head_diameter,center=true);   
  if(sunk_screw==true){   
   cylinder(h=screw_head_diameter/2,d1=0,d2=screw_head_diameter,center=true);    
  }    
 }       
}    

//cut off the base plate top rounding
module do_plate(){  
 if (base_enable==true){     
  if ( base_top_rounding==true){   
   intersection(){   
    plate();   
    translate([base_top_x_shift,base_top_y_shift,-base_top_radius+baseheight+baseborderheight])   
    if (show_cutting==true){     
     #sphere(base_top_radius,$fn=top_radius_accuracy);
    } else {
     scale([basescale_x,1,1])       
     sphere(base_top_radius,$fn=top_radius_accuracy);   
    }    
   }   
  } else {
   plate();
  }
 }
} 
     
// build the base plate and its border
module plate(){     
 union(){   
  linear_extrude(baseheight){   
   offset(limitedouteroff){
    scale([basescale_x,1,1])   
    rotate([0,0,baserotation])   
    circle(br,$fn=baseedges);   
   }
  }
  linear_extrude(finalborderheight){ 
   difference(){  
    offset(limitedouteroff){
     scale([basescale_x,1,1])   
     rotate([0,0,baserotation])     
     circle(br,$fn=baseedges);   
    } 
    offset(limitedinneroff){
     scale([basescale_x,1,1])    
     rotate([0,0,baserotation])   
     circle(br,$fn=baseedges);   
    }
   }
  }
 }     
} 

// build the hook body out of two extruded 2D svg shapes by intersection
module do_hook(){   
 if (enable_svg_hook==true){   
  translate([hook_x_shift,hook_y_shift,hook_z_shift])
  scale([hook_scale_x_factor,1,1])   
  rotate([-hook_x_rotate,0,0])   
  intersection(){ 
   rotate([0,90,90])  
   linear_extrude(200,center=true)
   scale([svg_hook_scale,svg_hook_scale])   
   import(xy_svg,center=true); 
   rotate([-90,90,270])  
   linear_extrude(200,center=true)
   scale([svg_hook_scale,svg_hook_scale]) 
   import(xz_svg,center=true);  
  }
 }
 if(enable_standard_hook==true) {
   translate([hook_x_shift,hook_y_shift,hook_z_shift])
   scale([hook_scale_x_factor,1,1])  
   rotate([-hook_x_rotate,0,0])  
   union(){  
    translate([0,0,(hook_length-hook_ball_diameter/2)/2])   
    cylinder(h=hook_length-hook_ball_diameter/2,d2=hook_shaft_diameter,d1=hook_tapered_diameter,center=true);   
    translate([0,0,hook_length-hook_ball_diameter/2])   
    if(hook_ball_diameter < hook_shaft_diameter) {
     minkowski(){
      cylinder(h=.01,d=hook_shaft_diameter-hook_ball_diameter,center=true);
      sphere(d=hook_ball_diameter);   
     }    
    } else {   
     sphere(d=hook_ball_diameter);   
    }    
    translate([0,0,-49.99])     
    cylinder(h=100,d=hook_tapered_diameter,center=true);       
   }    
 }    
}    

// build the svg body an roundthe top
module do_svg(){
 if (svg_top_rounding==true){
  intersection(){
   translate([svg_top_radius_x_shift+svg_x_shift,svg_top_radius_y_shift+svg_y_shift,-svg_top_radius+svg_topdist-.01])    
   rotate([0,0,svg_top_radius_rotate+svg_rotate])     
   scale([1,svg_tr_scale,1])   
   //translate([0,0,-svg_top_radius+svg_topdist-.01]) 
   if (show_cutting==true){    
    #sphere(svg_top_radius,$fn=top_radius_accuracy);
   } else {
    sphere(svg_top_radius,$fn=top_radius_accuracy);
   }
   svg_foot();
  }
 } else {
  svg_foot();   
 }  
} 

// build the text body an roundthe top
module do_text(){
 if((text_enable == true)&& (line_01 != "" )) {
  union(){   
   if (text_top_rounding==true){
    intersection(){
     translate([text_top_radius_x_shift+text_x_shift,text_top_radius_y_shift+text_y_shift,0])    
     rotate([0,0,text_top_radius_rotate+text_rotate])     
     scale([text_top_radius_x_scale,1,1])   
     translate([0,0,-text_top_radius+text_topdist-.01]) 
     if (show_cutting==true){    
      #sphere(text_top_radius,$fn=top_radius_accuracy);
     } else {
      sphere(text_top_radius,$fn=top_radius_accuracy);
     }
     textcore();
    }
   } else {
    textcore();   
   } 
  } 
 }
}

// text core (inner text)
module textcore(){
 union(){   
  linear_extrude(textlayer){  // the inner text - positive
   offset(0.01){  
    pattern_text();
   }   
  }
  text_foot();  
 }
}    

// make the svg 3d
module svg_foot(){                      // make the svg foot part    
 union(){   
  if (svg_import_enabled == true) {    
   if ((svg_rounding != 0) && (svg_import_enabled == true)) {
    linear_extrude(svg_thickness){   
     difference(){
      outersvg();    
      draw_svg();
     }     
    }
   }  
   linear_extrude(svglayer){ // the inner svg   
    offset(0.01){  
     draw_svg();   
    }    
   }
  }   
 }
}

// make the text 3d
module text_foot(){                      // make the outer text             
 linear_extrude(text_thickness){   
  difference(){
   outertext();    
   pattern_text();
  }
 }
}  

// create the 2d outer svg
module outersvg() { // make the base of the hook
 if ((svg_rounding != 0) && (svg_import_enabled == true)) { 
  offset(svg_rounding){  
   draw_svg(); 
  }
 }
}

// create the 2d outer text 
module outertext() {
 if ((text_rounding != 0) && (text_enable == true) && (line_01 != "" )) { 
  offset(text_rounding){  
   pattern_text();
  }
 } 
} 


// Flip text to wrong reading
module fliptext(){
 if(text_flip == true){
  rotate([180,0])
   pattern_text_lines();
 } else {
  pattern_text_lines();
 }    
}            
    
// Draw Text (2D)
module pattern_text_lines(){
 line_space_val = pattern_text_size/2 + text_line_space;
 line_count = (len(text_set));
 for (line = [ 0 : 1 : line_count-1 ]){
  line_height_val = pattern_text_size + line_space_val;
  temp_value = (line_count-1)/2;
  translate([0,(-pattern_text_size/2) + (line_height_val*temp_value) ,0])
  translate([0, (line_height_val * -line) ,0]) 
  text(text = text_set[line], valign = "baseline", halign = "center", size = pattern_text_size, font = pattern_text_font_val,spacing=text_spacing);
 }
}

// Transform Text (2D)
module pattern_text(text_rounding = 0){
 translate([text_x_shift,text_y_shift,0]) rotate(text_rotate){
  if(text_rounding == 0){
   fliptext();
  } else {
   difference(){
    offset(text_rounding){
     fliptext();
    }
    fliptext();
   }
  }
 } 
}

// Draw SVG (2D)
module draw_svg(svg_rounding = 0){
 if(svg_flip == true){
    rotate([0,180])
    show_svg(svg_rounding);
 } else {
    show_svg(svg_rounding);
 }    
}
    
// create the svg 2d base
module show_svg(svg_rounding){    
 translate([svg_x_shift,svg_y_shift,0]) rotate(svg_rotate){
  if(svg_rounding == 0){
   scale([file_scale,file_scale]){
    import(file_to_import, center = true);
   }
  } else {
   difference(){
    offset(svg_rounding){
     scale([file_scale,file_scale]){
      import(file_to_import, center = true);
     }
    }
    scale([file_scale,file_scale]){
     import(file_to_import, center = true);
    }
   }
  }
 } 
}

